import { VSBuffer } from "@codingame/monaco-vscode-api/vscode/vs/base/common/buffer";
import { CancellationToken } from "@codingame/monaco-vscode-api/vscode/vs/base/common/cancellation";
import { Disposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { UriComponents } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { IUriIdentityService } from "@codingame/monaco-vscode-api/vscode/vs/platform/uriIdentity/common/uriIdentity.service";
import { ITestProfileService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/testing/common/testProfileService.service";
import { ITestResultService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/testing/common/testResultService.service";
import { ITestService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/testing/common/testService.service";
import { CoverageDetails, ExtensionRunTestsRequest, IFileCoverage, ITestItem, ITestMessage, ITestRunProfile, ITestRunTask, ResolvedTestRunRequest, TestControllerCapability, TestResultState, TestsDiffOp } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/testing/common/testTypes";
import { IExtHostContext } from "../../services/extensions/common/extHostCustomers.js";
import { ILocationDto, ITestControllerPatch, MainThreadTestingShape } from "@codingame/monaco-vscode-api/vscode/vs/workbench/api/common/extHost.protocol";
export declare class MainThreadTesting extends Disposable implements MainThreadTestingShape {
    private readonly uriIdentityService;
    private readonly testService;
    private readonly testProfiles;
    private readonly resultService;
    private readonly proxy;
    private readonly diffListener;
    private readonly testProviderRegistrations;
    constructor(extHostContext: IExtHostContext, uriIdentityService: IUriIdentityService, testService: ITestService, testProfiles: ITestProfileService, resultService: ITestResultService);
    $markTestRetired(testIds: string[] | undefined): void;
    $publishTestRunProfile(profile: ITestRunProfile): void;
    $updateTestRunConfig(controllerId: string, profileId: number, update: Partial<ITestRunProfile>): void;
    $removeTestProfile(controllerId: string, profileId: number): void;
    $addTestsToRun(controllerId: string, runId: string, tests: ITestItem.Serialized[]): void;
    $appendCoverage(runId: string, taskId: string, coverage: IFileCoverage.Serialized): void;
    $startedExtensionTestRun(req: ExtensionRunTestsRequest): void;
    $startedTestRunTask(runId: string, task: ITestRunTask): void;
    $finishedTestRunTask(runId: string, taskId: string): void;
    $finishedExtensionTestRun(runId: string): void;
    $updateTestStateInRun(runId: string, taskId: string, testId: string, state: TestResultState, duration?: number): void;
    $appendOutputToRun(runId: string, taskId: string, output: VSBuffer, locationDto?: ILocationDto, testId?: string): void;
    $appendTestMessagesInRun(runId: string, taskId: string, testId: string, messages: ITestMessage.Serialized[]): void;
    $registerTestController(controllerId: string, _label: string, _capabilities: TestControllerCapability): void;
    $updateController(controllerId: string, patch: ITestControllerPatch): void;
    $unregisterTestController(controllerId: string): void;
    $subscribeToDiffs(): void;
    $unsubscribeFromDiffs(): void;
    $publishDiff(controllerId: string, diff: TestsDiffOp.Serialized[]): void;
    $runTests(req: ResolvedTestRunRequest, token: CancellationToken): Promise<string>;
    $getCoverageDetails(resultId: string, taskIndex: number, uri: UriComponents, token: CancellationToken): Promise<CoverageDetails.Serialized[]>;
    dispose(): void;
    private withLiveRun;
}
